@@ -41,6 +41,12 @@ if [ ! -f "${SRC}" ]; then
 	fi
 	export FOSSILTAG
 
+	if [ -d 'buildsrc' ]; then
+		# Override here to avoid downloading tarball from Fossil if we
+		# have a particular tree already available.
+		use_fossil='0'
+	fi
+
 	if [ "${use_fossil}" = "1" ]; then
 		(
 			cd src || exit 1
@@ -93,9 +99,20 @@ if [ ! -f "${SRC}" ]; then
 			rm -rf "${workdir}"
 		) || exit 1
 	else
-		rm -f "${SRC}.tmp"
-		wget -O "${SRC}.tmp" "${SRCURL}" || exit 1
-		mv "${SRC}.tmp" "${SRC}"
+		if [ ! -d 'buildsrc' ]; then
+			rm -f "${SRC}.tmp"
+			wget -O "${SRC}.tmp" "${SRCURL}" || (
+				echo '  Unable to download source code for Tcl.' >&4
+				echo "  Attempted to run:  wget -O \"${SRC}.tmp\" \"${SRCURL}\"" >&4
+				echo "  Got:"
+				wget -O "${SRC}.tmp" "${SRCURL}" 2>&1 | sed 's@^@    @' >&4
+
+				echo '  Aborting Tcl -- further packages will likely also fail.' >&4
+
+				exit 1
+			) || exit 1
+			mv "${SRC}.tmp" "${SRC}"
+		fi
 	fi
 fi
 
@@ -132,7 +149,16 @@ fi
 		fi
 	done
 
-	for dir in unix win macosx __fail__; do
+	tryfirstdir=''
+	if [ "$(uname -s)" = "Darwin" ]; then
+		tryfirstdir='macosx'
+	fi
+
+	for dir in "${tryfirstdir}" unix win macosx __fail__; do
+		if [ -z "${dir}" ]; then
+			continue
+		fi
+
 		if [ "${dir}" = "__fail__" ]; then
 			# If we haven't figured out how to build it, reject.
 
@@ -144,6 +170,9 @@ fi
 
 		cd "${BUILDDIR}/${dir}" || exit 1
 
+		# Remove broken pre-generated Makfiles
+		rm -f GNUmakefile Makefile makefile
+
 		echo "Running: ./configure --disable-shared --with-encoding=utf-8 --prefix=\"${INSTDIR}\" ${CONFIGUREEXTRA}"
 		./configure --disable-shared --with-encoding=utf-8 --prefix="${INSTDIR}" ${CONFIGUREEXTRA}
 
