@@ -22,12 +22,39 @@ export ITCLVERS SRC SRCURL BUILDDIR OUTDIR INSTDIR
 rm -rf 'build' 'out' 'inst'
 mkdir 'build' 'out' 'inst' || exit 1
 
+# Determine if Threads is even needed
+(
+	TCL_VERSION="unknown"
+	if [ -f "${TCLCONFIGDIR}/tclConfig.sh" ]; then
+		source "${TCLCONFIGDIR}/tclConfig.sh"
+	fi
+
+	if echo "${TCL_VERSION}" | grep '^8\.[45]$' >/dev/null; then
+		# Threads may be required for Tcl 8.4 and Tcl 8.5
+
+                exit 0
+        fi
+
+	if [ "${TCL_VERSION}" = "unknown" ]; then
+		# If we dont know what version of Tcl we are building, build
+		# Threads just in case.
+
+		exit 0
+	fi
+
+	# All other versions do not require Threads
+	echo "Skipping building Threads, not required for ${TCL_VERSION}"
+	exit 1
+) || exit 0
+
 if [ ! -f "${SRC}" ]; then
 	mkdir 'src' >/dev/null 2>/dev/null
 
-	rm -f "${SRC}.tmp"
-	wget -O "${SRC}.tmp" "${SRCURL}" || exit 1
-	mv "${SRC}.tmp" "${SRC}"
+	if [ ! -d 'buildsrc' ]; then
+		rm -f "${SRC}.tmp"
+		wget -O "${SRC}.tmp" "${SRCURL}" || exit 1
+		mv "${SRC}.tmp" "${SRC}"
+	fi
 fi
 
 (
@@ -35,15 +62,18 @@ fi
 
 	if [ ! -d '../buildsrc' ]; then
 		gzip -dc "../${SRC}" | tar -xf -
-	else    
+	else
 		cp -rp ../buildsrc/* './'
 	fi
 
 	cd "${BUILDDIR}" || exit 1
+	echo "Running: ./configure --enable-shared --disable-symbols --prefix=\"${INSTDIR}\" --exec-prefix=\"${INSTDIR}\" --with-tcl=\"${TCLCONFIGDIR}\" ${CONFIGUREEXTRA}"
 	./configure --enable-shared --disable-symbols --prefix="${INSTDIR}" --exec-prefix="${INSTDIR}" --with-tcl="${TCLCONFIGDIR}" ${CONFIGUREEXTRA}
 
+	echo "Running: ${MAKE:-make}"
 	${MAKE:-make} || exit 1
 
+	echo "Running: ${MAKE:-make} install"
 	${MAKE:-make} install
 
 	mkdir "${OUTDIR}/lib" || exit 1
