@@ -41,8 +41,21 @@ if [ ! -f "${SRC}" ]; then
 	use_fossil='0'
 	if echo "${TCLVERS}" | grep '^cvs_' >/dev/null; then
 		use_fossil='1'
+
+		FOSSILTAG=$(echo "${TCLVERS}" | sed 's/^cvs_//g')
+		if [ "${FOSSILTAG}" = "HEAD" ]; then
+			FOSSILTAG="trunk"
+		fi
 	elif echo "${TCLVERS}" | grep '^fossil_' >/dev/null; then
 		use_fossil='1'
+
+		FOSSILTAG=$(echo "${TCLVERS}" | sed 's/^fossil_//g')
+	fi
+
+	if [ -d 'buildsrc' ]; then
+		# Override here to avoid downloading tarball from Fossil if we
+		# have a particular tree already available.
+		use_fossil='0'
 	fi
 
 	if [ "${use_fossil}" = "1" ]; then
@@ -57,8 +70,13 @@ if [ ! -f "${SRC}" ]; then
 			mkdir "${workdir}" || exit 1
 			cd "${workdir}" || exit 1
 
-			wget -O "tmp-tk.tar.gz" "http://core.tcl.tk/tk/tarball/tk-fossil.tar.gz?uuid=${FOSSILDATE}" || rm -f 'tmp-tk.tar.gz'
-			gzip -dc "tmp-tk.tar.gz" | tar -xf -
+			wget -O "tmp-tk.tar.gz" "http://core.tcl.tk/tk/tarball/tk-fossil.tar.gz?uuid=${FOSSILTAG}" || rm -f 'tmp-tk.tar.gz'
+			gzip -dc "tmp-tk.tar.gz" | tar -xf - || rm -f 'tmp-tk.tar.gz'
+
+			if [ ! -s 'tmp-tk.tar.gz' ]; then
+				wget -O "tmp-tk.tar.gz" "http://core.tcl.tk/tk/tarball/tk-fossil.tar.gz?uuid=${FOSSILDATE}" || rm -f 'tmp-tk.tar.gz'
+				gzip -dc "tmp-tk.tar.gz" | tar -xf -
+			fi
 
 			mv "tk-fossil" "tk${TCLVERS}"
                         
@@ -68,9 +86,11 @@ if [ ! -f "${SRC}" ]; then
 			rm -rf "${workdir}"
 		)
 	else
-		rm -f "${SRC}.tmp"
-		wget -O "${SRC}.tmp" "${SRCURL}" || exit 1
-		mv "${SRC}.tmp" "${SRC}"
+		if [ ! -d 'buildsrc' ]; then
+			rm -f "${SRC}.tmp"
+			wget -O "${SRC}.tmp" "${SRCURL}" || exit 1
+			mv "${SRC}.tmp" "${SRC}"
+		fi
 	fi
 fi
 
@@ -119,7 +139,11 @@ fi
 		fi
 	done
 
-	for dir in unix win macosx win64 __fail__; do
+	for dir in "${TCLCONFIGDIRTAIL}" unix win macosx win64 __fail__; do
+		if [ -z "${dir}" ]; then
+			continue
+		fi
+
 		if [ "${dir}" = "__fail__" ]; then
 			exit 1
 		fi
@@ -136,6 +160,9 @@ fi
 
 		cd "${BUILDDIR}/${dir}" || exit 1
 
+		# Remove broken pre-generated Makfiles
+		rm -f GNUmakefile Makefile makefile
+
 		if [ "${dir}" = "win" ]; then
 			# Statically link Tk to Tclkit if we are compiling for
 			# Windows
